# $Header: sd_start.tcl,v 1.6 96/03/03 20:45:42 van Locked $ (LBL)
#
# NOTE: This file is provided for reference purposes.  The code
#       shown here is compiled into sd.  DON'T use this as your
#	sd startup file (~/.sd.tcl).
#
# tcl 'hooks' invoked when sd takes some action on a session.
#
# sd will invoke:
#	start_session	when the user asks to 'open' (start) a session
#	create_session	just after the user creates a new session
#	heard_session	when announcement for a session is first heard
#	delete_session	when the user or a timeout deletes a session
#
# When any of the above are invoked, the global array sd_sess
# contains all the information about the session to be started:
#   sd_sess(name)
#   sd_sess(description)
#   sd_sess(address)
#   sd_sess(ttl)
#   sd_sess(creator)
#   sd_sess(creator_id)
#   sd_sess(source_id)
#   sd_sess(arrival_time)
#   sd_sess(start_time)
#   sd_sess(end_time)
#   sd_sess(attributes)		(list of session attributes)
#   sd_sess(media)		(list of media names)
#
# For each media name there is an array containing the information
# about that media:
#   sd_$media(port)
#   sd_$media(conf_id)
#   sd_$media(attributes)	(list of media attributes)
#
# Media and session attributes are strings of the form "name" or
# "name:value".
#
# Some global state information is available in array sd_priv:
#   sd_priv(audio)		(= 0 if audio disabled with -a)
#   sd_priv(video)		(= 0 if video disabled with -v)
#   sd_priv(whiteboard)		(= 0 if wb disabled with -w)

proc start_session {} {
	global sd_sess sd_priv

	# invoke the appropriate start proc for each of the media
	# if such a proc exists and that media is enabled.
	foreach m $sd_sess(media) {
		if { [llength [info proc start_$m]] && $sd_priv($m) } {
			start_$m
		}
	}
}

set ipc_chan 0
proc get_channel { confid } {
        global ipc_tab ipc_chan
        if { [info exists ipc_tab($confid)] } {
                return $ipc_tab($confid)
        }
        incr ipc_chan
        if { $ipc_chan > 300 } {
                set ipc_chan 1
        }
        set ipc_tab($confid) $ipc_chan
        return $ipc_chan
}

proc start_audio {} {
        global sd_sess sd_audio sd_video
        set audiofmt ""
        set packetfmt "-n"
        set mode "-c"
	set recvonly ""
        foreach a $sd_audio(attributes) {
                case $a {
                        fmt:* { set audiofmt [string range $a 4 end] }
                        rtp { set packetfmt -r }
                        lecture  { set mode -l }
			recvonly { set recvonly -R }
			sendrecv { set recvonly -XrecvOnly=false }
                }
        }
        set confaddr [format "%s/%s/%s/%s/%s" $sd_sess(address) \
                $sd_audio(port) $sd_audio(conf_id) $audiofmt $sd_sess(ttl)]

        global vat
	if {[lsearch $sd_sess(media) video] >= 0} {
		set chan [get_channel $sd_sess(address)/$sd_video(port)]
		exec $vat -I $chan -C $sd_sess(name) $packetfmt $mode $confaddr &
	} else {
		exec $vat -C $sd_sess(name) $packetfmt $mode $confaddr &
	}
}

proc start_video {} {
        global sd_sess sd_video
        set videofmt "nv"
        foreach a $sd_video(attributes) {
                case $a {
                        fmt:* { set videofmt [string range $a 4 end] }
                }
        }
        case $videofmt {
                vic { }
		nv { }
		telesia { set videofmt ivs }
		ivs { }
                jpg {
                        global imm
                        exec nice $imm -p $sd_video(port) -I $sd_sess(address) \
                             -ttl $sd_sess(ttl) -n $sd_sess(name) &
			return
                }
		default {
			puts "sd: unknown video format: $videofmt"
			return
		}
        }
	global vic
	if {[lsearch $sd_sess(media) audio] >= 0} {
		set chan [get_channel $sd_sess(address)/$sd_video(port)]
		exec $vic -I $chan -A $videofmt -t $sd_sess(ttl) \
			-C $sd_sess(name) \
			$sd_sess(address)/$sd_video(port) &
	} else {
		exec $vic -A $videofmt -t $sd_sess(ttl) \
			-C $sd_sess(name) \
			$sd_sess(address)/$sd_video(port) &
	}
}

proc start_whiteboard {} {
	global sd_sess sd_whiteboard wb
	set orient -l
	set recvonly -r
	foreach a $sd_whiteboard(attributes) {
		case $a {
			orient:portrait { set orient -p }
			orient:landscape { set orient -l }
			orient:seascape { set orient +l }
			recvonly { set recvonly -r }
			sendrecv { set recvonly +r }
		}
	}
	exec $wb -t $sd_sess(ttl) -C wb:$sd_sess(name) $recvonly $orient \
		$sd_sess(address)/$sd_whiteboard(port) &
}

proc create_session {} {
	global sd_sess sd_priv
	set is_call 0
	foreach a $sd_sess(attributes) {
		case $a { 
			type:call { set is_call 1 }
		}
	}
	if { $is_call && $sd_priv(localaddr) == $sd_sess(source_id) } {
		start_session
	}
}

proc heard_session {} {
}

proc change_session {} {
}

proc delete_session {} {
}

# set up media option menus for new session window.

set sd_menu(audio) "fmt: pcm dvi gsm lpc4 pcm2 pcm4 dvi2 dvi4\nlecture\nvt"
set sd_menu(video) "fmt: vic nv ivs jpg"
set sd_menu(whiteboard) "orient: portrait landscape seascape\nsendrecv\nrecvonly"

# set up the command names

set vat vat
set vic vic
set nv nv
set ivs ivs
set imm imm
set wb wb
